local _
local L
local prefix = "NomiTab"
local debug
local guild
local playerName
local date
local libGraph
local graph
local color
local oldTab
local initiated

local function output(msg, omit)
	print((not omit and prefix .. " - " or "") .. msg)
end

local function dbg(msg)
	if debug then
		output(msg, 1)
	end
end

local function printHelp(type)
	if type and L[type] then
		output(L[type])
	else
		local help = {
				"'graph': show graph of throughput",
				"'compare': compare gains since <x> days ago (if no data present, use closest data)",
				"'help': display this text"
				}
		output("Available commands:")

		for i = 1, #help do
			output(help[i], 1)
		end
	end
end

local function getCmd(msg)
	if msg then
		local a,b,c = strfind(msg, "(%S+)")
		if a then
			return c, strsub(msg, b+2)
		else	
			return ""
		end
	end
end

local function initDate()
	guild = GetGuildInfo("player")

	if not guild then
		return
	end

	if not NomiTab_Data["data"][guild] then
		NomiTab_Data["data"][guild] = {}
	end

	if not NomiTab_Data["data"][guild][date] then
		NomiTab_Data["data"][guild][date] = {
					["\124cffffffff\124Hitem:74653:0:0:0:0:0:0:0:90:0:0\124h[Steamed Crab Surprise]\124h\124r"] = 0,
					["\124cffffffff\124Hitem:74646:0:0:0:0:0:0:0:90:0:0\124h[Black Pepper Ribs and Shrimp]\124h\124r"] = 0,
					["\124cffffffff\124Hitem:74656:0:0:0:0:0:0:0:90:0:0\124h[Chun Tian Spring Rolls]\124h\124r"] = 0,
					["\124cffffffff\124Hitem:74648:0:0:0:0:0:0:0:90:0:0\124h[Sea Mist Rice Noodles]\124h\124r"] = 0,
					["\124cffffffff\124Hitem:74650:0:0:0:0:0:0:0:90:0:0\124h[Mogu Fish Stew]\124h\124r"] = 0
				}
	end
end

local function getDate(ofsDay)
	ofsDay = tonumber(ofsDay)

	if not ofsDay or ofsDay < 0 then
		ofsDay = 0
	end

	return tonumber(_G["date"]("%y%m%d", time() - 86400 * ofsDay))		-- 86400 sec/24h
end

local function getLastDate()
	local tar, tot

	for i = 0, 365 do
		tar = getDate(i)

		if NomiTab_Data["data"][guild][tar] then
			tot = 0

			for item, count in pairs(NomiTab_Data["data"][guild][tar]) do
				tot = tot + count
			end

			if tot ~= 0 then
				return tar
			end
		end
	end

	return getDate(0)
end

local function updateDate()
	local old = date
	date = getDate()

	if not date or not guild or not NomiTab_Data["data"][guild] or not NomiTab_Data["data"][guild][date] then
		initDate()
	end
end

local function init()
	playerName = UnitName("player")
	date = 0

	if not NomiTab_Data then
		NomiTab_Data = {}
	end

	if not NomiTab_Data["nomiTab"] then
		NomiTab_Data["nomiTab"] = 1
	end

	if not NomiTab_Data["data"] then
		local temptab = NomiTab_Data["nomiTab"]
		NomiTab_Data["nomiTab"] = nil

		local temparr = NomiTab_Data
		NomiTab_Data = nil
		NomiTab_Data = {
					["nomiTab"] = temptab,
					["data"] = temparr
				}
	end

	updateDate()

	L = NomiTab_Loc["enUS"]
	NomiTab_Loc = nil

	NomiTab:UnregisterEvent("ADDON_LOADED")

	-----------------------------------------------------
	-----------------------------------------------------

	color = {
			["black"]	= {0.0, 0.0, 0.0},
			["red"]		= {1.0, 0.0, 0.0},
			["green"]	= {0.0, 1.0, 0.0},
			["blue"]	= {0.0, 0.0, 1.0},
			["yellow"]	= {1.0, 1.0, 0.0},
			["purple"]	= {1.0, 0.0, 1.0},
			["cyan"]	= {0.0, 1.0, 1.0},
			["orange"]	= {1.0, 0.5, 0.0},
			["greydark"]	= {0.3, 0.3, 0.3},
			["greylight"]	= {0.7, 0.7, 0.7},
			["white"]	= {1.0, 1.0, 1.0},
		}

	local h, w = 512, 512

	libGraph = LibStub:GetLibrary("LibGraph-2.0")

	graphFrame = CreateFrame("Frame", "NomiTabGraphFrame", UIParent)
	graphFrame:SetSize(w, h)
	graphFrame:SetPoint("CENTER", UIParent, "CENTER", 0, 0)
	graphFrame:Hide()

	local tex = graphFrame:CreateTexture("Background", "BACKGROUND")
	tex:SetAllPoints()
	tex:SetTexture(0, 0, 0, 0.5)

--	graph = libGraph:CreateGraphScatterPlot("NomiTabGraphFrameGraph", graphFrame, "TOPLEFT", "TOPLEFT", 0, 0, w, h)
	graph = libGraph:CreateGraphLine("NomiTabGraphFrameGraph", graphFrame, "TOPLEFT", "TOPLEFT", 0, 0, w, h)

	local f

	f = graphFrame:CreateFontString(nil, "ARTWORK", "GameFontNormal")
	f:SetPoint("TOP", graphFrame, "BOTTOM", w / 2, -8)
	f:SetSize(100, 13)
	graphFrame.xlabelmax = f

	f = graphFrame:CreateFontString(nil, "ARTWORK", "GameFontNormal")
	f:SetPoint("TOP", graphFrame, "BOTTOM", -w / 2, -8)
	f:SetSize(100, 13)
	graphFrame.xlabelmin = f

	f = graphFrame:CreateFontString(nil, "ARTWORK", "GameFontNormal")
	f:SetPoint("RIGHT", graphFrame, "LEFT", -8, h / 2)
	f:SetSize(100, 13)
	f:SetJustifyH("RIGHT")
	graphFrame.ylabelmax = f

	f = graphFrame:CreateFontString(nil, "ARTWORK", "GameFontNormal")
	f:SetPoint("RIGHT", graphFrame, "LEFT", -8, -h / 2)
	f:SetSize(100, 13)
	f:SetJustifyH("RIGHT")
	graphFrame.ylabelmin = f

	---

	for i = 1, 5 do
		f = CreateFrame("Button", "NomiTabGraphFrameGraphLegend" .. i, graphFrame)
		f:SetPoint("LEFT", graphFrame, "RIGHT", 3, 67 - i * 17)
		f:SetSize(15, 15)

		tex = f:CreateTexture(nil, "BACKGROUND")
		tex:SetAllPoints()
		tex:SetTexture(0, 0, 0, 0.6)

		f.Background = tex

		f.itemLink = "\124cffffffff\124Hitem:74650:0:0:0:0:0:0:0:90:0:0\124h[Mogu Fish Stew]\124h\124r"

		f:SetScript("OnEnter", function(self)
						GameTooltip:SetOwner(self, "ANCHOR_RIGHT")
						GameTooltip:SetHyperlink(self.itemLink)
						CursorUpdate(self)
					end)
		f:SetScript("OnLeave", function(self)
						GameTooltip:Hide()
						ResetCursor()
					end)
		f:SetScript("OnUpdate", function(self)
						if GameTooltip:IsOwned(self) then
							GameTooltip:SetOwner(self, "ANCHOR_RIGHT")
							GameTooltip:SetHyperlink(self.itemLink)
						end
						CursorOnUpdate(self)
					end)
		f:SetScript("OnClick", function(self)
						HandleModifiedItemClick(self.itemLink)
					end)

		graphFrame["legend" .. i] = f
	end

	f = nil
	tex = nil

	-----------------------------------------------------
	-----------------------------------------------------
end

local function dateToString(last, title)
	return ((last == getDate(0) and ((not title and "earlier t" or "T") .. "oday")) or (last == getDate(1) and ((not title and "y" or "Y") .. "esterday")) or ((last .. ""):sub(5,6) .. "/" .. (last .. ""):sub(3,4)))
end

local function compareData(dataA, dataB, last, store)
	local diff = {}
	local msg = {}
	local balance = 0

	msg[1] = "Results since " .. dateToString(last) .. ":"

	for itemLink, count in pairs(dataB) do
		diff[itemLink] = count - dataA[itemLink]

		if diff[itemLink] ~= 0 then
			msg[#msg + 1] = itemLink .. " : " .. dataA[itemLink] .. "|cff" .. ((diff[itemLink] > 0 and "00ff00+") or (diff[itemLink] < 0 and "ff0000-")) .. math.abs(diff[itemLink]) .. "|r"
		end

		balance = balance + diff[itemLink]
	end

	if not msg[2] then
		msg[2] = "No change!"
	end

	output(msg[1])
	for i = 2, #msg do
		output(msg[i], 1)
	end

	if balance ~= 0 then
		output("|cffff0000The sum is not zero! Difference of |r" .. balance .. "|cffff0000.|r", 1)
	end

	if store then
		for itemLink, count in pairs(dataB) do
			NomiTab_Data["data"][guild][date][itemLink] = dataB[itemLink]
		end
	end
end

local function readTab(tab)
	local data = {}

	updateDate()

	local texturePath, itemCount, locked, itemLink

	for slot = 1, 98 do
		texturePath, itemCount, locked = GetGuildBankItemInfo(tab, slot)
		itemLink = GetGuildBankItemLink(tab, slot)

		if itemLink and NomiTab_Data["data"][guild][date][itemLink] then
			data[itemLink] = (data[itemLink] or 0) + itemCount
		end
	end

	local last = getLastDate()

	compareData(NomiTab_Data["data"][guild][last], data, last, true)
end

local function command(msg)
	local cmd, subCmd = getCmd(msg)

	if cmd == "graph" then
		if graphFrame:IsShown() then
			graphFrame:Hide()
			return
		end

		local last, number = getCmd(subCmd)

		local from, to

		last = tonumber(last)
		number = tonumber(number)

		if last and last > 0 and last < 1000 then		-- show data from last to number (default 0 (today))
			updateDate()

			if not number or number >= last then
				number = 0
			end

			last = getDate(last)
			number = getDate(number)

			local points = {}
			local arr, tar, tot
			local num, max = 0, 0
			local avg = {}

			for index = last, number do
				if index % 100 == 32 then
					index = index + 69
				end
				if math.floor((index % 10000) / 100) == 13 then
					index = index + 88
				end

				arr = NomiTab_Data["data"][guild][index]

				if arr then
					tot = nil
					for k, v in pairs(arr) do
						if v and v > 0 then
							tot = 1
						end
					end

					if tot then
						if not from then
							from = index
						end
						to = index

						num = num + 1
						for key, count in pairs(arr) do
							if not points[key] then
								points[key] = {{},{}}
								avg[key] = 0
							end

							tar = #points[key][1] + 1
							points[key][1][tar] = tar
							points[key][2][tar] = count

							avg[key] = ((avg[key] * (tar - 1)) + count) / tar

							if count > max then
								max = count
							end
						end
					end
				end
			end

			if tar < 3 then		-- too few elements - bugging with only 2 points
				output("Too few elements. Increase offset!")
				return
			end

			graphFrame:Show()

			graph:ResetData()

			graph:SetXAxis(1, num)
			graph:LockXMin(true)
			graph:LockXMax(true)
			graphFrame.xlabelmin:SetText(dateToString(from, true))
			graphFrame.xlabelmax:SetText(dateToString(to, true))

			graph:SetYAxis(0, max)
			graph:LockYMin(true)
			graph:LockYMax(true)
			graphFrame.ylabelmin:SetText(0)
			graphFrame.ylabelmax:SetText(max)

			graph:SetGridSpacing(1, max / 10)


			local tmp = {}
			for j = 1, max do
				tmp[j] = ""
			end
			for item, count in pairs(avg) do
				table.insert(tmp, count, item)
			end

			local ca, arr, i = {
					"red","green","blue","yellow","cyan"
					}, {}, 0
			for j = max, 1, -1 do
				arr = points[tmp[j]]
				if arr then
					i = i + 1
					graph:AddDataSeries(arr, color[ca[i]])
					graphFrame["legend" .. i].itemLink = tmp[j]
					graphFrame["legend" .. i].Background:SetTexture(color[ca[i]][1], color[ca[i]][2], color[ca[i]][3], 0.6)
				end
			end
		else
			printHelp("notlast")
		end
	elseif cmd == "compare" then
		local tar, number = getCmd(subCmd)
		local cmpDate

		tar = tonumber(tar)

		if tar then
			local oldArr

			tar = math.abs(tar)

			while true do
				cmpDate = getDate(tar)

				oldArr = NomiTab_Data["data"][guild][cmpDate]

				if oldArr then
					break
				end

				tar = tar - 1

				if tar == 0 then
					return
				end
			end

			compareData(oldArr, NomiTab_Data["data"][guild][date], cmpDate)
		end
	elseif cmd == "tab" then
		local tar, _ = getCmd(subCmd)

		tar = tonumber(tar)

		if tar and tar < 10 and tar > 0 then
			NomiTab_Data["nomiTab"] = tar
		end
	elseif cmd == "dumpData" then
		if subCmd == "all" then
			for date,arr in pairs(NomiTab_Data["data"][guild]) do
				print("---"..date.."---")
				for k,v in pairs(arr) do
					print(k,v)
				end
			end
		else
			for k,v in pairs(NomiTab_Data["data"][guild][date]) do
				print(k,v)
			end
		end
	elseif cmd == "test" then
		for date,arr in pairs(NomiTab_Data["data"][guild]) do
			print("-- " .. date .. " --")
			for k,v in pairs(arr) do
				print(k,v)
			end
		end
	else
		printHelp()
	end
end

function NomiTab_OnEvent(self, event, ...)
	if event == "ADDON_LOADED" and ... == "NomiTab" then
		init()
	elseif event == "GUILDBANKFRAME_OPENED" then
		oldTab = GetCurrentGuildBankTab()

		if GetCurrentGuildBankTab() ~= NomiTab_Data["nomiTab"] then
			SetCurrentGuildBankTab(NomiTab_Data["nomiTab"])
		end

		QueryGuildBankTab(NomiTab_Data["nomiTab"])

		if not initiated then
			initiated = true
			output("Initial pull of contents is slow. Please reopen bankframe.")
		else
			readTab(NomiTab_Data["nomiTab"])
		end

		SetCurrentGuildBankTab(oldTab)
	elseif event == "GUILDBANKBAGSLOTS_CHANGED" then
		
	end
end

function NomiTab_OnLoad()
	local regs = {
			"ADDON_LOADED",
			"GUILDBANKFRAME_OPENED",
			"GUILDBANKBAGSLOTS_CHANGED"
		}

	for i = 1, #regs do
		NomiTab:RegisterEvent(regs[i])
	end

	SLASH_NOMITAB1 = "/nt"
	SLASH_NOMITAB2 = "/nomi"
	SLASH_NOMITAB3 = "/nomitab"
	SlashCmdList["NOMITAB"] = command
end